@@ -55,6 +55,8 @@ GetText.add('Connection Manager','${escape(_("Connection Manager"))}')
 GetText.add('Connection restored','${escape(_("Connection restored"))}')
 GetText.add('Connections','${escape(_("Connections"))}')
 GetText.add('Cookies','${escape(_("Cookies"))}')
+GetText.add('Copy','${escape(_("Copy"))}')
+GetText.add('Copy Magnet URI','${escape(_("Copy Magnet URI"))}')
 GetText.add('Copy of .torrent files to:','${escape(_("Copy of .torrent files to:"))}')
 GetText.add('Copyright 2007-2018 Deluge Team','${escape(_("Copyright 2007-2018 Deluge Team"))}')
 GetText.add('Create','${escape(_("Create"))}')
@@ -92,6 +94,8 @@ GetText.add('Encryption','${escape(_("Encryption"))}')
 GetText.add('Error','${escape(_("Error"))}')
 GetText.add('Expand All','${escape(_("Expand All"))}')
 GetText.add('External IP Address','${escape(_("External IP Address"))}')
+GetText.add('Failed to download torrent','${escape(_("Failed to download torrent"))}')
+GetText.add('Failed to upload torrent','${escape(_("Failed to upload torrent"))}')
 GetText.add('File','${escape(_("File"))}')
 GetText.add('File Browser','${escape(_("File Browser"))}')
 GetText.add('Filename','${escape(_("Filename"))}')
