@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2011 Nick Lanham <nick@afternight.org>
 #
@@ -7,11 +6,9 @@
 # See LICENSE for more details.
 #
 
-from __future__ import unicode_literals
-
 import logging
 
-from deluge.common import is_ip
+from deluge.common import is_interface
 from deluge.decorators import overrides
 from deluge.i18n import get_languages
 from deluge.ui.client import client
@@ -94,11 +91,12 @@ class BasePreferencePane(BaseInputPane, BaseWindow, PopupsHandler):
                     )
                 elif ipt.name == 'listen_interface':
                     listen_interface = ipt.get_value().strip()
-                    if is_ip(listen_interface) or not listen_interface:
+                    if is_interface(listen_interface) or not listen_interface:
                         conf_dict['listen_interface'] = listen_interface
                 elif ipt.name == 'outgoing_interface':
                     outgoing_interface = ipt.get_value().strip()
-                    conf_dict['outgoing_interface'] = outgoing_interface
+                    if is_interface(outgoing_interface) or not outgoing_interface:
+                        conf_dict['outgoing_interface'] = outgoing_interface
                 elif ipt.name.startswith('proxy_'):
                     if ipt.name == 'proxy_type':
                         conf_dict.setdefault('proxy', {})['type'] = ipt.get_value()
