@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (C) 2008 Andrew Resch <andrewresch@gmail.com>
 #
@@ -7,15 +6,13 @@
 # See LICENSE for more details.
 #
 
-from __future__ import division, unicode_literals
-
 import json
 import logging
 import os.path
 
 import gi  # isort:skip (Required before Gtk import).
 
-gi.require_version('Gtk', '3.0')  # NOQA: E402
+gi.require_version('Gtk', '3.0')
 
 # isort:imports-thirdparty
 from gi.repository import Gio, Gtk
@@ -84,7 +81,7 @@ def cell_progress(column, cell, model, row, data):
 
 class FilesTab(Tab):
     def __init__(self):
-        super(FilesTab, self).__init__('Files', 'files_tab', 'files_tab_label')
+        super().__init__('Files', 'files_tab', 'files_tab_label')
 
         self.listview = self.main_builder.get_object('files_listview')
         # filename, size, progress string, progress value, priority, file index, icon id
